@@ -2,7 +2,7 @@
 #' @description Function to estimate and validate a CUB model for given ordinal responses, with covariates for
 #'  explaining the feeling component.
 #' @aliases fastcub0q
-#' @usage fastcub0q(m,ordinal,W,starting=NULL,maxiter,toler,iterc=3,invgen=TRUE, verbose=FALSE)
+#' @usage fastcub0q(m,ordinal,W,starting=NULL,maxiter,toler,iterc=3,invgen=TRUE)
 #' @param m Number of ordinal categories
 #' @param ordinal Vector of ordinal responses
 #' @param W Matrix of selected covariates for explaining the feeling component, not including intercept
@@ -11,19 +11,19 @@
 #' @param toler Fixed error tolerance for final estimates 
 #' @param iterc Iteration from which the acceleration strategy starts 
 #' @param invgen Logical: should the recursive formula for the inverse of the information matrix be considered? (Default is TRUE)
-#' @param verbose Logical: should messages about  acceleration steps be printed out? (Default is FALSE)
 #' @import stats graphics
 #' @return An object of the class "fastCUB"
 #' @keywords internal 
 
 
 #######################
-#######################
-fastcub0q<-function(m,ordinal,W,starting=NULL,maxiter,toler,iterc=3,invgen=TRUE, verbose=FALSE){
+fastcub0q<-function(m,ordinal,W,starting=NULL,maxiter,toler,iterc=3,invgen=TRUE){
   tt0<-proc.time()
   n<-length(ordinal)
   W<-as.matrix(W)
- 
+  # if (ncol(W)==1){
+  #   W<-as.numeric(W)
+  # }
   q<-NCOL(W)
   aver<-mean(ordinal)
   WW<-cbind(1,W)                   
@@ -38,17 +38,15 @@ fastcub0q<-function(m,ordinal,W,starting=NULL,maxiter,toler,iterc=3,invgen=TRUE,
   }
   
   ##############################################################
-  beta0jj<-log(paijj/(1-paijj))
-  loglikjj<-loglikcub0q(m,ordinal,W,beta0jj,gamajj)
+  loglikjj<-loglikcub0q(m,ordinal,W,paijj,gamajj)
   # ********************************************************************
   # ************* E-M algorithm for CUB(0,q) ***************************
   # ********************************************************************
   nniter<-1
-  iterc2<-iterc
   while(nniter<=maxiter){
     
-    thetaold<-c(beta0jj,gamajj)
-    paijj<-1/(1+exp(-beta0jj))
+    thetaold<-c(paijj,gamajj)
+    
     loglikold<-loglikjj
     vettn<-bitgama(m,ordinal,W,gamajj)
     ttau<-1/(1+(1-paijj)/(m*paijj*vettn)) 
@@ -59,24 +57,31 @@ fastcub0q<-function(m,ordinal,W,starting=NULL,maxiter,toler,iterc=3,invgen=TRUE,
     optimgama<-optim(gama,effe01,esterno01=cbind(ttau,ordinal,WW),m=m) 
     ################################################################
     gamajj<-optimgama$par
-    paijj<-sum(ttau)/n                   
-    beta0jj<-log(paijj/(1-paijj))
+    paijj<-sum(ttau)/n                    #updated pai estimate
     
-    ## aggiungo
-    thetanew<-c(beta0jj,gamajj)
-    param<-thetanew
-    loglikjj<-loglikcub0q(m,ordinal,W,beta0jj,gamajj)
     
-   
-    if (nniter>= iterc2){
+    csii<-logis(W,gamajj)
+    
+    thetanew<-c(paijj,gamajj)
+    
+    
+    ai<- ordinal - 1 -  (m-1)*(1-csii)
+    
+    #ai<- (m-1)*(1-csii) - (ordinal-1)
+    if (nniter>= iterc){
       
+      param<-thetanew
       
-      csii<-logis(W,gamajj)
-      ai<- ordinal - 1 -  (m-1)*(1-csii)
+      loglikold<-loglikcub0q(m,ordinal,W,paijj,gamajj)
       
-      vettn<-as.numeric(bitgama(m,ordinal,W,gamajj)   )
+      # M1<-vcScorec(ttau,param,ai,Y=NULL,W)
+      # M2<-vcScore(ttau,param,ai,Y=NULL,W)
+      # 
+      # InfC<-Ic(ttau,ordinal,param,ai,Y=NULL,W)
+      vettn<-bitgama(m,ordinal,W,gamajj)
       ttau<-1/(1+(1-paijj)/(m*paijj*vettn)) 
-      dc<-decomp(ttau,ordinal,m,param,ai,Y=NULL,W=W)      
+      dc<-decomp(ttau,ordinal,m,param,ai,Y=NULL,W=W)
+      
       M1<-dc$vcScorec
       M2<-dc$vcScore
       InfC<-dc$Ic
@@ -99,77 +104,31 @@ fastcub0q<-function(m,ordinal,W,starting=NULL,maxiter,toler,iterc=3,invgen=TRUE,
       if (invgen==TRUE){
         Iinv<- invmatgen(D,H,listE)
       } else {
-        Iinv<- try(solve(D+H),silent=TRUE)
+        Iinv<- solve(D+H)
       }     
       
-      if (any(class(Iinv)=="try-error")){
-        Iinv<-matrix(NA,nrow=nrow(D),ncol=nrow(D))
-      }
-      
-      if (any(is.infinite(Iinv)) | any(is.nan(Iinv))){
-        Iinv<-matrix(NA,nrow=nrow(D),ncol=nrow(D))
-      }
-      
-      if (all(class(Iinv)!="try-error") && !any(is.na(Iinv))){
-        
-        if (verbose==TRUE){
-          print("accelerating")
-        }
-   
-        inv<-Iinv%*%InfC
-        dk<- thetanew-thetaold
-        
-        thetastar<- thetanew + inv%*%dk
-        loglikold2<-loglikcub0q(m,ordinal,W,beta0jj,gamajj)
-        beta0jj2<-thetastar[1]; gamajj2<-thetastar[2:(q+2)]
-        loglikjj2<-loglikcub0q(m,ordinal,W,beta0jj2,gamajj2)
-        
-        if ( is.nan(loglikjj2)) {
-          Iinv<-matrix(NA,nrow=nrow(D),ncol=nrow(D))
-         if (verbose==TRUE) {cat("not accelerating at iter",nniter,"\n")}
-          
-          nniter <- nniter + 1
-          next
-        } else if (loglikjj2 < loglikold2){
-          Iinv<-matrix(NA,nrow=nrow(D),ncol=nrow(D))
-          if (verbose ==TRUE) {cat("not accelerating at iter",nniter,"\n")}
-          
-          nniter <- nniter + 1
-          next
-        } else {
-          beta0jj<-beta0jj2; gamajj<-gamajj2; paijj<-1/(1+exp(-beta0jj))            #%
-          loglikold<-loglikold2
-          loglikjj<-loglikjj2
-        
-        }
-        
-        
-      }  else {
-        if (verbose == TRUE) {cat("not accelerating at iter",nniter,"\n")}
-        Iinv<-matrix(NA,nrow=nrow(D),ncol=nrow(D))
-        
-        iterc2<-iterc2 + 10
-        nniter <- nniter + 1
-        next
-      }
+      inv<-Iinv%*%InfC
+      dk<- thetanew-thetaold
       
+      thetastar<- thetanew + inv%*%dk
       
+      paijj<-thetastar[1]; gamajj<-thetastar[2:(q+2)]
       
       
     }
     
     
+    loglikjj<-loglikcub0q(m,ordinal,W,paijj,gamajj)## needed for nlm version
+    # print(c(nniter,paijj,gamajj,loglikjj)); #OPTIONAL PRINTING OF ITERATIONS
     testll<-abs(loglikjj-loglikold)
-   
     
     
     if(testll<=toler) break else {loglikold<-loglikjj}
     nniter<-nniter+1
   }
   
-  pai<-paijj;  gama<-gamajj;  loglik<-loglikjj; stime<-c(paijj,gamajj)
   if (nniter < iterc) {
-    param<-c(beta0jj,gamajj)
+    param<-c(paijj,gamajj)
     
     vettn<-bitgama(m,ordinal,W,gamajj)
     ttau<-1/(1+(1-paijj)/(m*paijj*vettn))
@@ -196,45 +155,37 @@ fastcub0q<-function(m,ordinal,W,starting=NULL,maxiter,toler,iterc=3,invgen=TRUE,
     if (invgen==TRUE){
       Iinv<- invmatgen(D,H,listE)
     } else {
-      Iinv<- try(solve(D+H),silent=TRUE)
+      Iinv<- solve(D+H)
     }     
   }
   
-  if (any(class(Iinv)=="try-error")){
-    Iinv<- matrix(NA,nrow=length(param),ncol=length(param))
-  }
-  
   
   if(any(is.na(Iinv))==TRUE){
     warning("ATTENTION: NAs produced")
     varmat<-vmatLouis<-matrix(NA,nrow=nrow(Iinv),ncol=nrow(Iinv))
-    se<-rep(NA,nrow(Iinv))
   } else {
     if(det(Iinv)<=0){  
       warning("ATTENTION: Variance-covariance matrix NOT positive definite")
       varmat<-vmatLouis<-matrix(NA,nrow=nrow(Iinv),ncol=nrow(Iinv))
-      se<-rep(NA,nrow(Iinv))
     } else {
       varmat<-vmatLouis<-Iinv
-      varianze<-diag(vmatLouis)
-      paihat<-stime[1]
-      beta0hat<-log(paihat/(1-paihat))
-      sepai<-sqrt(varianze[1])*exp(-beta0hat)/((1+exp(-beta0hat))^2)
-      se<-c(sepai,sqrt(varianze[-1]))
-      
     }
   }
   durata<-proc.time()-tt0;durata<-durata[1];
   
-  
+  pai<-paijj;  gama<-gamajj;  loglik<-loglikjj; stime<-c(paijj,gamajj)
   ####################################################################
   AICCUB0q<- -2*loglik+2*(q+2)
   BICCUB0q<- -2*loglik+log(n)*(q+2)
   ####################################################################
- 
+  # Compute asymptotic standard errors of ML estimates
+  ####################################################################
   
   
-  results<-list('estimates'=stime,'time'=durata,'se'=se,'loglik'=loglik,'niter'=nniter,'varmat'=varmat,'BIC'=BICCUB0q,'vmatLouis'=vmatLouis)
- 
+  results<-list('estimates'=stime,'time'=durata,
+                'loglik'=loglik,'niter'=nniter,'varmat'=varmat,
+                'BIC'=BICCUB0q,'vmatLouis'=vmatLouis)
+  # class(results)<-"cub"
   return(results)
 }
+
